# -*- coding: utf-8 -*-
# @author: leesoar

"""postcard"""

MAIL_ADDR = {
    "tom": "pop.tom.com",
    "gmail": "pop.gmail.com",
    "yahoo": "pop.mail.yahoo.com",
    "china": "pop.china.com",
    "21cn": "pop.21cn.com",
    "163": "pop.163.com",
    "126": "pop.126.com",
    "qq": "pop.qq.com",
    "sina": "pop3.sina.com.cn",
}



# -*- coding: utf-8 -*-
# @author: leesoar
# @email: secure@tom.com
# @email2: employ@aliyun.com

import argparse

from postcard import __version__

parser = argparse.ArgumentParser(description=f"postcard", prog="postcard", add_help=False)
parser.add_argument('-v', '--version', action='version', version=__version__, help='Get version of postcard')
parser.add_argument('-h', '--help', action='help', help='Show help message')
parser.parse_args()


def run():
    return "Powered by leesoar.com"



# -*- coding: utf-8 -*-
# @author: leesoar
# @email: secure@tom.com
# @email2: employ@aliyun.com

from os.path import join, isfile
from os import walk
import io
import os
import sys
from shutil import rmtree
from setuptools import setup, Command

from postcard import __version__


def read_file(filename):
    with open(filename) as fp:
        return fp.read().strip()


def read_requirements(filename):
    return [line.strip() for line in read_file(filename).splitlines()
            if not line.startswith('#')]


NAME = 'postcard'
FOLDER = 'postcard'
DESCRIPTION = 'A POP3/SMTP client.'
URL = 'https://leesoar.com'
EMAIL = 'secure@tom.com'
AUTHOR = 'leesoar'
REQUIRES_PYTHON = '>=3.3.0'

REQUIRED = read_requirements('requirements.txt')

here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION


def package_files(directories):
    paths = []
    for item in directories:
        if isfile(item):
            paths.append(join('..', item))
            continue
        for (path, directories, filenames) in walk(item):
            for filename in filenames:
                paths.append(join('..', path, filename))
    return paths


class UploadCommand(Command):
    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')

        sys.exit()


setup(
    name=NAME,
    version=__version__,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    # packages=find_packages(exclude=('tests',)),
    packages=['postcard'],
    install_requires=REQUIRED,
    include_package_data=True,
    license='MIT',
    entry_points={
        'console_scripts': ['postcard = postcard.run:run']
    },
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    # $ setup.py publish support.
    cmdclass={
        'upload': UploadCommand,
    },
    keywords=[
        'mail', 'email', 'postal', 'postcard',
        'mailbox', 'postbox',
    ],
)



# -*- coding: utf-8 -*-
# @author: leesoar

"""POP3 client"""

import base64
import poplib
from email.parser import Parser

from postcard import setting


__all__ = ["Pop3"]


class Pop3(object):
    def __init__(self):
        self.user = None
        self.pwd = None
        self._client = None
        self.pop_addr = None
        self.parser = Parser()

    def login(self, user, pwd, pop_addr=None):
        """Login current mailbox"""
        self.user = user
        self.pwd = pwd
        self.pop_addr = pop_addr or self.pop_addr

        if not self.pop_addr:
            self.pop_addr = self._set_pop_addr_auto(user)

        self._client = poplib.POP3(self.pop_addr)
        self._client.user(self.user)
        self._client.pass_(self.pwd)
        return self._client

    def close(self):
        self._client.quit()

    @staticmethod
    def get_addr_map():
        return setting.MAIL_ADDR

    def _set_pop_addr_auto(self, user):
        domain = user.rsplit(".", maxsplit=1)[0].split("@")[-1]
        pop_addr = self.get_addr_map().get(domain)
        if pop_addr and not self.pop_addr:
            self.pop_addr = pop_addr
        return self.pop_addr

    def get_client(self):
        """Get pop3 client"""
        return self._client

    def how_many(self) -> int:
        """Return mailbox's email count"""
        return self._client.stat()[0]

    def __check_and_decode(self, mail_info: dict) -> dict:
        """Check encoding and decode mail's content"""
        content_encoding = mail_info.get("content-transfer-encoding", "").lower()

        try:
            if content_encoding == "base64":
                mail_info["content"] = base64.b64decode(mail_info["content"]).decode(errors="ignore")
            elif content_encoding == "quoted-printable":
                pass
        except Exception:
            pass

        return mail_info

    def retrieve(self, index_: int = None) -> dict:
        """Retrieve mail at index"""
        index_ = index_ or self.how_many()
        line_array = self._client.retr(index_)[1]
        delimiter = line_array.index(b"")
        content = (b"".join(line_array[delimiter:])).decode(errors="ignore")

        def __convert(x: bytes) -> list:
            k, v = x.decode(errors="ignore").split(":", 1)
            return [k.lower().strip(), v.lower().strip().lstrip("[").rstrip("]")]

        mail_info = dict(map(__convert, filter(lambda x: b":" in x and b"\t" not in x, line_array)))
        mail_info.update(content=content)
        return self.__check_and_decode(mail_info)

    def delete(self, index_: int = None):
        """Delete mail at index"""
        index_ = index_ or self.how_many()
        self._client.dele(index_)

    def process(self, user=None, pwd=None, pop_addr=None):
        """process decorator

        Args:
            user: mailbox account
            pwd: mailbox password
            pop_addr: current mailbox's pop3 addr
        Usage:
            pop = Pop3()

            @pop.process(user="xxx", pwd="xxx")
            def get_content():
                content = pop.retrieve()["content"]
                log.debug(content)
        """
        user = user or self.user
        pwd = pwd or self.pwd
        pop_addr = pop_addr or self.pop_addr

        assert None not in [user, pwd], \
            "You should give your info."

        def decorator(func):
            def wrapper(*args, **kwargs):
                self.login(user, pwd, pop_addr)
                ret = func(*args, **kwargs)
                self.close()
                return ret
            return wrapper
        return decorator
